@@ -81,6 +81,7 @@ class Profile(object):
         self.created = created or time or now
         self.time = time or now
         self.profile_id = profile_id
+        self.profile_ids = []
 
         if self.domain is None:
             raise ValueError('domain should not be None')
@@ -149,6 +150,23 @@ class Profile(object):
         props2 = other.properties.copy()
         props3 = {}
 
+        profile_ids = []
+        if hasattr(self, 'profile_id') and self.profile_id:
+            profile_ids.append(self.profile_id)
+        if hasattr(other, 'profile_id') and other.profile_id:
+            profile_ids.append(self.profile_id)
+        if hasattr(self, 'profile_ids') and self.profile_ids:
+            profile_ids.extend(self.profile_ids)
+        if hasattr(other, 'profile_ids') and other.profile_ids:
+            profile_ids.extend(other.profile_ids)
+
+        if not hasattr(self, '_nsum'):
+            self._nsum = 1
+        if not hasattr(other, '_nsum'):
+            other._nsum = 1
+
+        nsum = self._nsum + other._nsum
+
         data_types = set(props1.keys()).union(set(props2.keys()))
 
         for data_type in data_types:
@@ -170,6 +188,8 @@ class Profile(object):
                                  init=False)
 
         profile.properties = props3
+        profile.profile_ids = profile_ids
+        profile._nsum = nsum
 
         return profile
 
@@ -720,8 +740,11 @@ class Profile(object):
         else:
             data = [x.decode() if isinstance(x, bytes) else x for x in data]
             data = dict(zip(data[::2], data[1::2]))
-            return cls.from_flat_data(data=data,
-                                      user_id=user_id)
+            profile = cls.from_flat_data(data=data,
+                                         user_id=user_id)
+            if not key is None:
+                profile.profile_id = cls.profile_id_from_key(key)
+            return profile
 
     @property
     def flat_data(self):
